#!/usr/bin/env python2.6
#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

from optparse import OptionParser
import model.dataFolder
import urllib, sys, tarfile, ConfigParser, os

usage = "usage: %prog [options]"
parser = OptionParser(usage=usage)
parser.add_option("-d", "--datafolder", help="folder to store data, you can specify 'root', 'home' or a direct path. The default is home", default="home")
(options, args) = parser.parse_args()
    
if os.path.exists(os.path.join(os.path.dirname(__file__), 'portable')):
    options.datafolder = "root"
model.dataFolder.setDataFolder(options.datafolder)

iniName = os.path.basename(model.dataFolder.updateIniUrl)
# Inform the user about download progress
def reporter(blocks, blockSize, fileSize):
	global progress
	percentage = float(blocks * blockSize) / float(fileSize) * 100
	if(percentage > progress):
		sys.stdout.write(".")
		sys.stdout.flush()
		progress += 10

# Download, extract and delete archives to the proper destination
def handleUpdate(name, url, destination):
	global progress

	print 'Downloading ' + name
	progress = 0
	baseName = os.path.basename(url)
	urllib.urlretrieve(url, os.path.join(model.dataFolder.dataFolder, baseName) , reporter)
	print '\nExtracting ' + name
	archive = tarfile.open(os.path.join(model.dataFolder.dataFolder, baseName), 'r:bz2')
	archive.extractall(os.path.join(model.dataFolder.dataFolder, destination))
	archive.close()
	os.unlink(os.path.join(model.dataFolder.dataFolder, baseName))

# Downloads the latest ini file and checks the version of each file
# Initiates an update if needed
def checkIni():
	update = False
	
	if not os.path.exists(model.dataFolder.dataFolder): os.mkdir(model.dataFolder.dataFolder)
	if os.path.exists(os.path.join(model.dataFolder.dataFolder, iniName)):
		iniOld = open(os.path.join(model.dataFolder.dataFolder, iniName))
	else:
		iniOld = open(os.path.join(model.dataFolder.dataFolder, iniName), 'w+')

	try:
		iniNew = urllib.urlopen(model.dataFolder.updateIniUrl)
	except:
		print "Couldn't update. The update server is probably down."
		sys.exit(-10)

	configOld = ConfigParser.ConfigParser()
	configOld.readfp(iniOld)

	configNew = ConfigParser.ConfigParser()
	try:
		configNew.readfp(iniNew)
	except:
		print "Couldn't update. The update server is probably down."
		sys.exit(-10)

	for section in configNew.sections():
		try:
			if configNew.get(section, 'version') > configOld.get(section, 'version'):
				update = True
				handleUpdate(section, configNew.get(section, 'url'), configNew.get(section, 'destination'))
		except ConfigParser.NoSectionError:
			update = True
			handleUpdate(section, configNew.get(section, 'url'), configNew.get(section, 'destination'))
	
	iniOld.close()
	iniNew.close()
	if update:
		urllib.urlretrieve(model.dataFolder.updateIniUrl, os.path.join(model.dataFolder.dataFolder, iniName))
		print 'Wrote new ini file.'

# Start update process
checkIni()

print 'All files up to date!'
